${HEADER}

'''
Created on ${DATE_TIME}

@author: ${AUTHOR}
'''

import gtk


from ${NAME}.common import version

version.getInstance().submitRevision("$Revision: 37 $")


LOGO_PIXBUF = gtk.gdk.pixbuf_new_from_file("${NAME}-logo.png")


def fill_widget_with_logo(centerview):
    pixbuf = LOGO_PIXBUF
    for i in range(1,3):
        hbox = gtk.HBox()
        for j in range(1,5):                
            image = gtk.Image()
            image.set_from_pixbuf(pixbuf)
            hbox.add(image)
        centerview.add(hbox)
        



                    


${HEADER}

'''
Created on ${DATE_TIME}

@author: ${AUTHOR}
'''

import hildon
import gobject
import dbus
import gtk
import webbrowser
import sys, traceback
import logging

import gdata.projecthosting.client
import gdata.projecthosting.data
import gdata.gauth
import gdata.client
import gdata.data
import atom.http_core
import atom.core

import threading
from Queue import *

from ${NAME}.gui.gtk.utils import LOGO_PIXBUF

from ${NAME}.common import version

version.getInstance().submitRevision("$Revision: 131 $")


class Async_Exception_Handler():
    def __init__(self):
        self._exception_fifo = None
        self._exception_signal_handler_id = None

    def start_async_exception_handler(self):
        self._exception_fifo = Queue(0)
        self._exception_signal_handler_id = gobject.timeout_add(1000, self._receive_exception_signal) 
    

    def stop_async_exception_handler(self):
        gobject.source_remove(self._exception_signal_handler_id)

    def send_exception_signal(self):
        '''
        Thread other than Gtk main, must call this method to post an exception
        '''
        self._exception_fifo.put(sys.exc_info())

    def _receive_exception_signal(self):
        '''
        This method take an exception from the FIFO list of exception and
        display the corresponding GUI
        '''
        try:
            type, value, tb = self._exception_fifo.get_nowait()
            self._default_exception_handler(type, value, tb,gtk_sync=True)
        except Empty:
            pass
        return True 


    def default_exception_handler(self, gtk_sync = False):
        '''
        Do appropriate action when the core throw a Exception.
        Currently simply display a banner message.
        '''
        type, value,tb = sys.exc_info()             
        self._default_exception_handler(type, value, tb, gtk_sync = gtk_sync)
                        
    def _default_exception_handler(self, type, value, tb, gtk_sync = False):
        logging.exception("!!! UNHANDLED EXCEPTION !!!")
        if gtk_sync :
            gtk.gdk.threads_enter()
        show_banner_information("An exception occured, please report the bug")
        window = BugReportView(type, value, tb)
        program = hildon.Program.get_instance() 
        program.add_window(window)
        window.show_all()
        if gtk_sync : 
            gtk.gdk.threads_leave()
    
ASYNC_EXCEPTION_HANDLER_INSTANCE = Async_Exception_Handler()


def not_yet_implemented(gtk_sync = False):
    if gtk_sync :
            gtk.gdk.threads_enter()
    show_banner_information("not yet implemented")
    if gtk_sync : 
            gtk.gdk.threads_leave()

def show_note_information(mess, gtk_sync = False):
       if gtk_sync :
            gtk.gdk.threads_enter()
       parent = hildon.WindowStack.get_default().peek()
       note = hildon.hildon_note_new_information(parent,mess)                 
       response = gtk.Dialog.run(note)
       if gtk_sync :
            gtk.gdk.threads_leave()


def show_banner_information(mess, gtk_sync = False):
       if gtk_sync :
            gtk.gdk.threads_enter()   
       parent = hildon.WindowStack.get_default().peek()
       banner= hildon.hildon_banner_show_information(parent,"", mess)
       banner.set_timeout(2000)  
       if gtk_sync :
            gtk.gdk.threads_leave()   

       

def _show_about_dialog():
    window = AboutView()
    program = hildon.Program.get_instance()     
    program.add_window(window)
    window.show_all()

def show_about_dialog(widget, data):
       '''
       Show an information dialog about the program
       '''
       call_handled_method(_show_about_dialog)


def open_browser_on( url):
        bus = dbus.SystemBus()
        browser = dbus.Interface(bus.get_object('com.nokia.osso_browser', '/com/nokia/osso_browser/request'), 'com.nokia.osso_browser')
        browser.load_url(url)




        
      
       
def call_handled_method(method, *arg, **kwarg):
       '''
       This utility function catch error on a gentle way.
       Warning *only* GTK thread can use it.
       ''' 
       try:
            method(*arg, **kwarg)
       except :
            # this is an unkown exception
            ASYNC_EXCEPTION_HANDLER_INSTANCE.default_exception_handler()


class ${NAME_CAP}StackableWindow(hildon.StackableWindow):
    '''
    general design of any GUI of this app
    '''
    
    centerview = None;
    bottomButtons = None;
    
    def __init__(self, title="Pyjama", async_init=False, raise_stop_flag=None):
        '''
        Paremeter:
            - title : window title
            - async_init: True call the init_center_view in a separate thread. Default: False
            - raise_stop_flag: if async_init is True, défine the function to call when raise a stop request on the synchroneous thread
        '''
        hildon.StackableWindow.__init__(self)
        
        self.program = hildon.Program.get_instance() 
        self.set_title(title)
   
        self.menu = hildon.AppMenu()                
        self.init_menu(self.menu)
        
        aboutMenuBtn = hildon.GtkButton(gtk.HILDON_SIZE_AUTO);
        aboutMenuBtn.set_label("About");
        aboutMenuBtn.connect("clicked", show_about_dialog, None)
        self.menu.append(aboutMenuBtn)

        
        self.menu.show_all()
        self.set_app_menu(self.menu)  
        
   
        container = gtk.VBox();                
                  
        self.centerview = gtk.VBox() 
        
        if async_init:
            async_thread = AsyncTask(self, raise_stop_flag, self.init_center_view, centerview)
            async_thread.start()            
        else:
            self.init_center_view(self.centerview)
        
        pannable_area = hildon.PannableArea()
        pannable_area.set_property('mov_mode',hildon.MOVEMENT_MODE_BOTH)
        pannable_area.add_with_viewport(self.centerview)
        
        self.bottomButtons = gtk.HBox(homogeneous=True)
        
        self.init_bottom_button(self.bottomButtons)
        

        bottomAlign = gtk.Alignment(0.5,0,0,0)
        bottomAlign.add(self.bottomButtons)
        
        container.pack_start(pannable_area)
        container.pack_start(bottomAlign, expand=False)
        
        self.add(container)


    def init_menu(self, menu):
        '''
        This hook should be implemented by subclass to add window
        specific menu item
        '''
        pass;

    def init_center_view(self, centerview):
        '''
        This hook should be implemented by subclass to
        add widget in the centerview which is both instance
        and attribute variable for convenience
        '''
        pass;
    
    def init_bottom_button(self, bottomButtons):
        '''
        This hook should be implemented by subclass to
        add button in the bottom button which is both instance
        and attribute variable for convenience
        '''
        pass;

    def add_button(self, button):
        '''
        goodies to add a button to this view
        '''
        self.bottomButtons.pack_start(button,True,True,0)
    
    def create_button(self, name, value=None):
        '''
        goodies to create a button for this view
        ''' 
        return hildon.Button(gtk.HILDON_SIZE_THUMB_HEIGHT, hildon.BUTTON_ARRANGEMENT_HORIZONTAL, name, value)

    def justifyLeft(self, widget):
          leftAlign = gtk.Alignment(0,0.5,0,0)
          leftAlign.add(widget)
          return leftAlign
      
    def justifyRight(self, widget):
          leftAlign = gtk.Alignment(1,0.5,0,0)
          leftAlign.add(widget)
          return leftAlign

class AboutView(${NAME_CAP}StackableWindow):
    '''
    This view show a fancy about dialog
    '''


    def init_center_view(self, centerview):
        
        pixbuf = LOGO_PIXBUF
        image = gtk.Image()
        image.set_from_pixbuf(pixbuf)
        centerview.add(image)
        
        centerview.add(gtk.Label("${NAME_CAP} - " + version.getInstance().getVersion()))
        centerview.add(gtk.Label("${SUMMARY}"))
        centerview.add(gtk.Label("by ${AUTHOR} - ${PROJECT_URL}"))


    def init_bottom_button(self, bottomButtons):
        blogButton = self.create_button("Blog")
        blogButton.connect("clicked", self.on_blog_clicked_event, None)
        self.add_button(blogButton)
        
        siteButton = self.create_button("Site")
        siteButton.connect("clicked", self.on_site_clicked_event, None)
        self.add_button(siteButton)
        
        groupsiteButton = self.create_button("Groups")
        groupsiteButton.connect("clicked", self.on_group_clicked_event, None)
        self.add_button(groupsiteButton)
        
        licenceButton = self.create_button("Licence")
        licenceButton.connect("clicked", self.on_licence_clicked_event, None)
        self.add_button(licenceButton)

    def on_licence_clicked_event(self, widget, data):
        dialog = gtk.Dialog()
        dialog.set_transient_for(self)
        dialog.set_title("Copyright information")        
        dialog.add_button("Ok", gtk.RESPONSE_OK)
        gpl = hildon.TextView()
        gpl_licence = """
$DESCRIPTION

$COPYRIGHT

$LICENCE
"""
        buffer = gtk.TextBuffer()
        buffer.set_text(gpl_licence)        
        gpl.set_buffer(buffer)
        gpl.set_property('editable', False)
        
        pannable_area = hildon.PannableArea()
        pannable_area.set_property('mov_mode',hildon.MOVEMENT_MODE_BOTH)
        pannable_area.set_property('size-request-policy', hildon.SIZE_REQUEST_CHILDREN)
        pannable_area.add_with_viewport(gpl)
        dialog.vbox.add(pannable_area)
        dialog.show_all()
        dialog.run()
        dialog.destroy()

    def on_blog_clicked_event(self, widget, data):                                
        call_handled_method(open_browser_on,"${PROJECT_URL}")
    
    def on_site_clicked_event(self, widget, data):
        call_handled_method(open_browser_on,"${SITE_URL}")
         
    def on_group_clicked_event(self, widget, data):
        call_handled_method(open_browser_on,"${GROUP_URL}")


class BugReportView(${NAME_CAP}StackableWindow):
    '''
    This view show the bug and give the user the opportunity to report it
    '''
    type = None
    value = None
    traceback = None
    
    submit_issue_callback = None
    
    _body = None
    _subject = None
    
    def __init__(self, type, value, traceback):
        self.type = type
        self.value = value
        self.traceback = traceback       
        ${NAME_CAP}StackableWindow.__init__(self, title="Bug reporting") 

    def init_center_view(self, centerview):
       
        subjectLbl = gtk.Label("Subject")
        centerview.pack_start(self.justifyLeft(subjectLbl), False)
        self._subject = hildon.Entry(gtk.HILDON_SIZE_FULLSCREEN_WIDTH)
        self._subject.set_placeholder("enter a subject")
        self._subject.set_text(str(self.type) + " : " + str(self.value))
        centerview.pack_start(self._subject, False)
        contentLbl = gtk.Label("Content")
        centerview.pack_start(self.justifyLeft(contentLbl), False)
        self._body = hildon.TextView()
        self._body.set_placeholder("enter the message here")
        self._body.set_wrap_mode(gtk.WRAP_WORD)
        stacktrace = traceback.format_exception(self.type, self.value, self.traceback)
        buf = self._body.get_buffer()
        for line in stacktrace:
            end = buf.get_end_iter()
            buf.insert(end, line, len(line))
        centerview.add(self._body)
        return ${NAME_CAP}StackableWindow.init_center_view(self, centerview)


    def init_bottom_button(self, bottomButtons):
        post = self.create_button("Post issue", None)
        post.connect("clicked", self.on_post_button_clicked, self)
        self.add_button(post)                     
        return ${NAME_CAP}StackableWindow.init_bottom_button(self, bottomButtons)

    def on_post_button_clicked(self, widget, view):
        buffer = view._body.get_buffer()
        body = buffer.get_text(buffer.get_start_iter(), buffer.get_end_iter())
        subject =  view._subject.get_text()
        self._submit_issue(subject, body)


    def _submit_issue(self, title, description):
        # check credential
        try:
            parent = hildon.WindowStack.get_default().peek()
            dialog = hildon.LoginDialog(parent)
            dialog.set_message("Gmail account required")            
            response = dialog.run()
            username = dialog.get_username()
            password = dialog.get_password()
            dialog.destroy()
            if response == gtk.RESPONSE_OK:
                try:
                    issues_client = gdata.projecthosting.client.ProjectHostingClient()
                    issues_client.client_login(username, password,"${NAME}", "code")
                    versionInstance = version.getInstance()
                    versionStr = versionInstance.getVersion()
                    revisionStr = versionInstance.getRevision()
                    labels = ['Type-Defect', 'Priority-High', 'Version-' + versionStr, 'Revision-' + revisionStr]
                    issues_client.add_issue("${NAME}", title, description, "tbressure", labels=labels)
                except:                    
                    show_banner_information("failed to send issue")
                    logging.exception("Failed to report the previous issue due to")
                else:
                    show_banner_information("issue sent")
            else:
                show_banner_information("bug report cancelled")
        finally:
            hildon.WindowStack.get_default().pop_1()


class AsyncTask(threading.Thread):
    '''
    This is a gtk aware asynchroneous task.
    '''
    def __init__(self, window, stop_hook,  method, *argv, **kwarg):
            self.task_name = "${NAME_CAP}-AsyncTask"
            threading.Thread.__init__(self,name=self.task_name)
            self.method = method   
            self.argv = argv
            self.kwarg = kwarg
            self.window = window        
            self.stop_hook = stop_hook
            if self.window:
                window.connect("destroy", self._destroy_async_window,  None)

    def _destroy_async_window(self,  widget, data):
         self.send_stop_signal()
                     
        
    def run(self):            
            logging.debug("%s thread running...." % self.task_name)              
            if self.window:
                gtk.gdk.threads_enter()
                hildon.hildon_gtk_window_set_progress_indicator(self.window, 1)
                gtk.gdk.threads_leave()      
            try :                         
                self.cancellableAction()
            finally:
                if self.window:
                    gtk.gdk.threads_enter()
                    hildon.hildon_gtk_window_set_progress_indicator(self.window, 0)
                    gtk.gdk.threads_leave()                            
            logging.debug("%s thread finished" % self.task_name)

    def cancellableAction(self):
            try:
                self.method(*self.argv, **self.kwarg)
            except StopSignalException, sse:
                logging.info("%s aborted by software" % self.task_name)
            except Exception, e:
                logging.error("uncaught exception in %s \n%s" %(self.task_name,str(e)))
                # this thread is different fromgtk one so we must call this function
                # to post the current exception
                ASYNC_EXCEPTION_HANDLER_INSTANCE.send_exception_signal()
            else:
                logging.debug("%s terminated" % self.task_name)
            
    def send_stop_signal(self):
        if self.stop_hook:
            self.stop_hook()
        
class StopSignalException(Exception):
    pass



${HEADER}

'''
Created on ${DATE_TIME}

@author: ${AUTHOR}
'''


from ..common import version 

version.getInstance().submitRevision("$Revision: 131 $")

class Settings():
    '''
    Represents the settings for Gnatirac
    '''
   
    def __init__(self):
        pass



${HEADER}

'''
Created on ${DATE_TIME}

@author: ${AUTHOR}
'''

from ..common import version 

version.getInstance().submitRevision("$Revision: 131 $")

class ${NAME_CAP}CaptchaException(Exception):
    '''
    Login failure because google required a capcha
    '''            
    
    def __init__(self, token, url, doc_storage=None):
        self.captcha_token = token
        self.captcha_url = url
        self.doc_storage = doc_storage
        
    def get_token(self):
        return self.captcha_token
    
    def get_url(self):
        return self.captcha_url
    
    def get_storage(self):
        return self.doc_storage


${HEADER}

'''
Created on ${DATE_TIME}

@author: ${AUTHOR}
'''

import logging
import pickle
import os
import os.path

from model import *

from ..common import version 

version.getInstance().submitRevision("$Revision: 131 $")


'''
Facade functions
'''

def get_pyjama_storage_dir():
    '''
    Compute the application storage dir.
    This is an utility function to retrieve the directory where pyjama can
    store any file like settings or cached data.
    '''
    storage = os.path.expanduser("~")    
    storage = os.path.join(storage, ".pyjama")
    return storage



class pyjama(object):
    '''
    Main class of the Program. The GUI use this class like a Facade to any core functions.
    '''
    
    def __init__(self):
        self.connected = False
        self._ensure_pyjama_conf_store()        
        self.settings = None        
        self.load_settings()        
        self.apply_settings()

    def get_pyjama_settings_file(self):
        storage = get_pyjama_storage_dir()
        storage = os.path.join(storage, "settings.pickle")
        return storage

    def load_settings(self):
        '''
        load the saved settings
        '''
        self._ensure_pyjama_conf_store()
        storage = self.get_pyjama_settings_file()
        try:           
            file = open(storage,'rb')
            self.settings = pickle.load(file)
            file.close()
        except IOError, EOFError:
            logging.warning("failed to load the settings")
            self.settings  = Settings()
            
    
    def save_settings(self):
        '''
        save the current settings
        '''
        self._ensure_pyjama_conf_store()
        storage = self.get_pyjama_settings_file()
        try:           
            file = open(storage,'wb')
            pickle.dump(self.settings,file)
            file.close()
        except IOError:
            logging.warning("failed to save the settings")

    
    def apply_settings(self):        
       pass
                               
    
    def _ensure_pyjama_conf_store(self):
        storage = get_pyjama_storage_dir()
        if os.path.exists(storage):
            pass
        else:
            os.makedirs(storage)
            
    
    '''
    Facade method
    '''
        
    
    def get_settings(self):
        return self.settings
    
    
  
    def get_pyjama_storage_dir(self):
        '''
        Storage location of pyajama
        '''
        return get_pyjama_storage_dir()
            



${HEADER}
'''
Created on ${DATE_TIME}

@author: ${AUTHOR}
'''


from ${NAME}.core import facade

from optparse import OptionParser

import logging

from ${NAME}.common import version 


version.getInstance().submitRevision("$Revision: 4 $")


LEVELS = {'debug': logging.DEBUG,
          'info': logging.INFO,
          'warning': logging.WARNING,
          'error': logging.ERROR,
          'critical': logging.CRITICAL}




def showGui():
    from ${NAME}.gui.hildon.${NAME}Gui import pyjamaGui 
    gui = pyjamaGui()
    gui.run()
    
def noGui():
    # directly use the facade
    # facade.xxxxx
    pass;
    
 

    
def run():
   versionManager = version.getInstance()
   usage = "%prog "
   str_version = "%prog " + versionManager.getVersion() + "(" + versionManager.getRevision() + ")"
   parser = OptionParser(usage=usage, version=str_version)
#
#   parser.add_option("-l","--login",action="store", dest="login", help="the login of the user without the @gmail")
#   parser.add_option("-p","--password",action="store", dest="password", help="the password for the given login")
#   parser.add_option("-d","--destination",action="store", dest="folder", help="the remote folder or album name")
#   parser.add_option("-r","--remove",action="store_true", dest="remove",  default=False, help="remove local file when uploaded")
#   parser.add_option("-f","--flatten",action="store_true", dest="flatten", default=False, help="subfolder are flattened to folder with name prefixed by given -d option value if any")
#   parser.add_option("-v","--verbose",action="store_true", dest="verbose", default=False, help="print verbose output of saving progress")
   parser.add_option("-g","--gui",action="store_true", dest="gui", default=False, help="show the gui")   
   parser.add_option("-l", "--log", action="store", type="string", dest="level_name", help="log level")
      
   (options, args) = parser.parse_args()
   
   level = LEVELS.get(options.level_name, logging.NOTSET)
   logging.basicConfig(level=level)

   if options.gui:   
       showGui()
   else:
       noGui()
    




if __name__ == '__main__':  
    run()



# -*- encoding: UTF-8 -*-

#    Pyjama is a python project bootstraper for Maemo5 development
#
#    Copyright (C) 2011  Thierry Bressure
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>

'''
Created on Jan 6, 2012

@author: maemo
'''

import os
import shutil
import logging
import string
import gtk

from ..common import version 

version.getInstance().submitRevision("$Revision: 131 $")


GPL_LICENCE = """This program is free software: you can redistribute
it and/or modify it under the terms of the GNU
General Public License as published by the Free
Software Foundation, either version 3 of the License,
or (at your option) any later version.

This program is distributed in the hope that it will
be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public
License for more details.

You should have received a copy of the GNU General
Public License along with this program. If not, see
<http://www.gnu.org/licenses/>
"""


def not_yet_implemented():
    raise NotImplementedError()

class Settings():
    '''
    Represents the settings for Gnatirac
    '''
   
    def __init__(self):
        pass

class Project():
    '''
    Project informations
    '''
            
    
    def __init__(self, type="default",
                  name="pyjama",
                  author=os.environ['LOGNAME'],
                  summary="pyjama generated project",
                  description="This a project generated by Pyjama with default layout" ,
                  inception=None,
                  logo_uri=None,
                  version="0.1.0"):
        self.type = type
        self.name = name
        self.author = author
        self.summary = summary
        self.description = description
        self.version = version
        if not inception:
            from datetime import date
            inception = date.today().year
        self.inception = inception
        self.logi_uri=logo_uri        
        self.file_encoding="-*- encoding: UTF-8 -*-" # python syntax to define source file encoding
        self.copyright = None
        self.header=None
        self.licence=GPL_LICENCE
        from datetime import datetime
        self.date_time = datetime.today().strftime("%d %B %Y %H:%M:%S")        
        self._generate_copyright()        
        self._generate_header() # generate full pythoh source file header        

    def _generate_copyright(self):
        copyleft = Copyright()
        SourceTransformer().process(copyleft, self, None) # no output dir because in memory processing
        self.copyright = copyleft.get_content()
        
    def _generate_header(self):
        header = Header()
        SourceTransformer().process(header, self, None) # no output dir because in memory processing
        self.header = header.get_content()
        
    def generate(self, output):
        '''
        Generate this project description.
        Parameter:
            - ouptut: the path where to generate the project named directory which contains project source 
        '''
        DefaultGenerator(self).generate(os.path.realpath(os.path.join(output,self.name)))
    
        


class IVisitableTemplate():
    '''
    An object that can be visited
    '''
    def acceptTransformer(self, visitor):
        '''
        Call this method to receive a visitor 
        '''
        not_yet_implemented()


class ITemplate():
    '''
    A template that can be modify by Transformer to generate project skeleton 
    '''
    
    def set_output(self, output):
        '''
        Modify the output i.e the location of the generated file. 
        Parameter:
            - ouput is a pathname file
        '''
        not_yet_implemented()

    def get_input(self):
        '''
        Return the location of the template. This is a pathname file
        '''
        not_yet_implemented()
    
    def get_content(self):
        '''
        Return the current value of the template.
        This value may have been modify by some Transformer since the template instanciation
        '''
        not_yet_implemented()
    
    def set_content(self, content):
        '''
        Modify the content of the template.
        '''
        not_yet_implemented()
        
    def generate(self):
        '''
        Generate the template using current content and current output        
        '''
        not_yet_implemented()
    

class Template(ITemplate, IVisitableTemplate):
    '''
    Base implementation of any template
    '''
    def __init__(self, id):
        logging.debug("instanciating %s id=%s" % ("Template", str(id)))
        self.id = id
        self.output = None        
        self.content = None

    def get_content(self):
        return self.content


    def set_content(self, content):
        self.content = content


    def set_output(self, output):
        self.output = output

    def _ensure_destination(self):
        dir_output = os.path.dirname(self.output)
        if not os.path.isdir(dir_output):
            os.makedirs(dir_output)
                
        
    
    
class MemoryTemplate(Template):
    '''
    A template based on memory value
    '''
    
    def __init__(self, id, value):
        Template.__init__(self, id)
        self.content = value

    def generate(self):
        # copy the content
        logging.debug("generating %s" % (self.output))
        self._ensure_destination()
        f = open(self.output,"wb")
        f.write(self.content)     
    
    def set_relative_output_file(self, root_output, rel_output_file):
         self.set_output(os.path.join(root_output, rel_output_file))   


class TextMemoryTemplate(MemoryTemplate):
    '''
    A memory template based on a text
    '''
    def __init__(self, id, value):
        MemoryTemplate.__init__(self, id, value)

    def acceptTransformer(self, visitor):
        return visitor.visit_TextMemoryTemplate(self)


         
class FileTemplate(Template):
    '''
    A template which is based on a file  
    '''
    
    def __init__(self, root, file):
        Template.__init__(self, file)            
        self.relative_input_file = "." + os.sep + file.partition(root)[2]      
        
    def _load_file_content(self):
        # load the text file content
        f =  open(self.get_input())
        self.set_content(f.read())
    
    def get_input(self):
        '''
        Return the template file input
        '''
        return self.id    
    
    def get_relative_input_file(self):
        return self.relative_input_file
    
    def set_relative_input_file(self, rel_input_file):
        self.relative_input_file = rel_input_file
            
    def set_relative_output_file(self, root_output, rel_output_file=None):
        '''
        Défine the output file of this template.
        Parameters:
            - root_output : the output dir
            - rel_output (optional) : the relative ouput file. Default: relative input file
        '''
        if not rel_output_file:
            rel_output_file = self.relative_input_file
        self.set_output(os.path.join(root_output, rel_output_file))                

    def generate(self):
        '''
        Copy input file to output file
        '''
        logging.debug("generating %s" % (self.output))
        self._ensure_destination()
        if self.content:
            # copy the content
            f = open(self.output,"wb")
            f.write(self.content)                
        else:
            shutil.copy(self.get_input(), self.output)


    def acceptTransformer(self, visitor):
        '''
        Tell the visitor to handle a FileTemplate
        '''
        return visitor.visit_fileTemplate(self)
        
    
    

class TextFileTemplate(FileTemplate):
    '''
    A file template representing text file (python source file...)
    '''
    def __init__(self, root, file):
        FileTemplate.__init__(self, root, file)
        self._load_file_content()

    def acceptTransformer(self, visitor):
        return visitor.visit_textFileTemplate(self)

    


class ImageFileTemplate(FileTemplate):
    '''
    A file template representing an image file.
    content attribute is a pixbuf
    '''
    def __init__(self, root, file):
        FileTemplate.__init__(self, root, file)
        self._load_file_content()
    
    def _load_file_content(self):
        # load the text file content
        f = open(self.get_input())
        self.set_content(gtk.gdk.pixbuf_new_from_file(self.get_input()))
        
    
    def generate(self):
        '''
        Copy input file to output file
        '''
        self._ensure_destination()
        if self.content:
            # copy the content
            if self.output.endswith(".png"):
                type = "png"
            elif self.output.enswith(".jpg"):
                type = "jpeg"
            else:
                type = "jpeg"
            self.content.save(self.output, type)                
        else:
            shutil.copy(self.get_input(), self.output)

class TemplateFactory():
    '''
    Factory of template according to filename
    '''
    
    TEXT_FILE_EXTENSION = [".txt", ".py", ".cfg", ".in", ".desktop"]
    
    IMAGE_FILE_EXTENSION = [".png", ".jpg"]
    
    def get_template(self, root, file):
        '''
        Return a template instance
        '''
        
        for extension in self.TEXT_FILE_EXTENSION:
            if file.endswith(extension):
                return TextFileTemplate(root, file)
        
        for extension in self.IMAGE_FILE_EXTENSION:
            if file.endswith(extension):
                return ImageFileTemplate(root, file)

        return FileTemplate(root, file)

class Copyright(TextMemoryTemplate, IVisitableTemplate):
    '''
    Copyright information
    '''            
    def __init__(self):
        MemoryTemplate.__init__(self, "Copyright",
                                value="Copyright (C) $INCEPTION  $AUTHOR")

class Header(TextMemoryTemplate, IVisitableTemplate):    
    '''
    Header of each source file. This portion of text before
    file specific information
    '''                      
    def __init__(self):
        MemoryTemplate.__init__(self, "Header",
                                value="""$FILE_ENCODING

$DESCRIPTION

$COPYRIGHT

$LICENCE
""")

    def acceptTransformer(self, visitor):
        return visitor.visit_HeaderTemplate(self)

    
        
          
class Licence(TextMemoryTemplate, IVisitableTemplate):
    '''
    The GNU Licence template
    '''
    
    def __init__(self):
        MemoryTemplate.__init__(self, "GNU Licence",
                                value=GPL_LICENCE)
       
        
        
class IVisitorTransformer():
    '''
    Visitor interface for Transfomer class
    '''        
    def visit_fileTemplate(self, template):
        '''
        handle visited FileTemplate
        '''
        not_yet_implemented()
        
    def visit_imageFileTemplate(self, template):
        '''
        handle visited ImageFileTemplate
        '''
        not_yet_implemented()
        
    def visit_textFileTemplate(self, template):
        '''
        handle visited textFileTemplate
        '''
        not_yet_implemented()
        
    def visit_HeaderTemplate(self, template):
        '''
        Handle visited HeaderTemplate
        '''
        not_yet_implemented()
        
    def visit_TextMemoryTemplate(self, template):
        '''
        Handle visited TextMemroyTemplate
        '''
        not_yet_implemented()
        
class ITransformer():
    '''
    Interface for Transformer class
    '''
    def process(self, template, project, root_output_dir):
        '''
        return a list of modified template
        ''' 
        not_yet_implemented()
        
class Transformer(ITransformer, IVisitorTransformer):
    
    
    def process(self, template, project, root_output_dir=None):
        self.project = project
        self.root_output_dir = root_output_dir
        return template.acceptTransformer(self)
    
    def _return_no_template(self):
        return []


class SourceTransformer(Transformer):

   

    # KEYWORD CONSTANTS
    HEADER = 'HEADER'
    COPYRIGHT = 'COPYRIGHT'
    DESCRIPTION = 'DESCRIPTION'
    SUMMARY = 'SUMMARY'
    AUTHOR = 'AUTHOR'
    LICENCE = 'LICENCE'
    FILE_ENCODING = 'FILE_ENCODING'
    NAME = 'NAME'
    INCEPTION = 'INCEPTION'
    VERSION = 'VERSION'
    DATE_TIME = 'DATE_TIME'
    
    
        
       
    def get_project_dico(self):
        dico = dict()
        dico[self.HEADER] = self.project.header
        dico[self.COPYRIGHT] = self.project.copyright
        dico[self.DESCRIPTION] = self.project.description
        dico[self.SUMMARY] = self.project.summary
        dico[self.AUTHOR] = self.project.author
        dico[self.LICENCE] = self.project.licence
        dico[self.FILE_ENCODING] = self.project.file_encoding
        dico[self.NAME] = self.project.name
        dico[self.NAME + "_CAP"] = self.project.name.capitalize()
        dico[self.INCEPTION] = self.project.inception
        dico[self.VERSION] = self.project.version
        dico[self.DATE_TIME] = self.project.date_time
        
        return dico       
    
    def _substitute_source(self, template, *argv, **kwarg):
        '''
        Transform the template with given keywords values
        '''
        template.set_content(string.Template(template.get_content()).safe_substitute(**kwarg))
        
    def _apply_project_defintion(self, template):
        '''
        Apply project definition on the template        
        '''
        d = self.get_project_dico()
        
        self._substitute_source(template, **d)

    def visit_fileTemplate(self, template):
        if template.get_relative_input_file().find("scripts/pyjama") > -1:
            # this file i snot detected as a text file so we manualy load its content 
            template._load_file_content()
            self._apply_project_defintion(template)          
            return [template]
        
        return self._return_no_template()


    def visit_imageFileTemplate(self, template):
        return self._return_no_template()


    def visit_textFileTemplate(self, template):
        self._apply_project_defintion(template)
        return [template]

    def visit_HeaderTemplate(self, template):
        self._apply_project_defintion(template)
        # header must be commented
        CommentOutTransformer().process(template, self.project, self.root_output_dir)
        return [template]        

    def visit_TextMemoryTemplate(self, template):
        self._apply_project_defintion(template)
        return [template]




 
    
class FilenameTransformer(Transformer):
    '''
    Apply transformation on file name
    '''

    def visit_fileTemplate(self, template):
        if template.get_relative_input_file().find("scripts/pyjama") > -1:
            template.set_relative_input_file(template.get_relative_input_file().replace("pyjama",self.project.name))            
            return [template]
        
        return self._return_no_template()


    def visit_imageFileTemplate(self, template):
        if template.get_relative_input_file().find("media/pyjama.png") > -1 or \
            template.get_relative_input_file().find("hildon/icons/48x48/pyjama.png") > -1 or \
            template.get_relative_input_file().find("hildon/icons/64x64/pyjama.png") > -1 or \
            template.get_relative_input_file().find("src/pyjama-logo.png") > -1:      
                template.set_relative_input_file(template.get_relative_input_file().replace("pyjama",self.project.name))
                return [template]
        return self._return_no_template()


    def visit_textFileTemplate(self, template):
        if template.get_relative_input_file().find("hildon/pyjama.desktop") > -1:
            template.set_relative_input_file(template.get_relative_input_file().replace("pyjama",self.project.name))            
            return [template]
        if template.get_relative_input_file().find("pyjama") > -1:
            template.set_relative_input_file(template.get_relative_input_file().replace("pyjama",self.project.name))            
            return [template]
        return self._return_no_template()

    def visit_HeaderTemplate(self, template):
        return self._return_no_template()


    def visit_TextMemoryTemplate(self, template):
        return self._return_no_template()





class CommentOutTransformer(Transformer):
    '''
    Comment out the processed template
    '''

    def visit_HeaderTemplate(self, template):
        result = ""        
        # read each line
        for line in template.get_content().splitlines():
            commented = "# " + line + "\n"
            result = result + commented
        template.set_content(result)
        return [template]
   

    

class LogoTransformer(Transformer):
    '''
    Apply transforamtion on image logo.
    '''

    def visit_fileTemplate(self, template):
        return self._return_no_template()


    def visit_imageFileTemplate(self, template):
        return self._return_no_template()


    def visit_textFileTemplate(self, template):
        return self._return_no_template()


    def visit_HeaderTemplate(self, template):
        return self._return_no_template()


    def visit_TextMemoryTemplate(self, template):
        return self._return_no_template()


class CopyTransformer(Transformer):
    '''
    Simply copy the template to the output path preserving input relative path
    '''

    def visit_fileTemplate(self, template):
        return self._copy_to_output_dir(template)


    def visit_imageFileTemplate(self, template):
        return self._copy_to_output_dir(template)


    def visit_textFileTemplate(self, template):
        return self._copy_to_output_dir(template)


    def visit_HeaderTemplate(self, template):
        return self._return_no_template()


    def visit_TextMemoryTemplate(self, template):
        return self._return_no_template()

    def _copy_to_output_dir(self, template):
        template.set_relative_output_file(self.root_output_dir)
        return [template]

   
   

TRANSFORMERS_PIPELINE = [SourceTransformer(), FilenameTransformer(), LogoTransformer(), CopyTransformer()]

class Generator():
    
    def __init__(self, project):
        self.project = project
        self.templateFactory = TemplateFactory()

    def generateTemplatePool(self, output,  aTemplatePool ):
        '''
        Process and generate template
        Parameter:
            - output: the directory root output
            - aTemplatePool : list of Template
        '''
       
        for transformer in TRANSFORMERS_PIPELINE:
            processedByTransformer = map(lambda aTemplate: transformer.process(aTemplate, self.project, output), aTemplatePool)
            # make a flat list of Template
            map(lambda x: aTemplatePool.extend(x), processedByTransformer)
            # remove duplicate template
            aTemplatePool = [template for template in set(aTemplatePool)]
        
        map(lambda x: x.generate(), aTemplatePool)


    def generate(self, output):
        '''
        Common generation 
        '''
        licence = Licence()
        licence.set_relative_output_file(output, "LICENCE")
        aTemplatePool = [licence]
        self.generateTemplatePool(output, aTemplatePool)
        

class DefaultGenerator(Generator):
    '''
    This class can generate a project skeleton given a project définition
    '''
    
    def __init__(self,project):
        Generator.__init__(self, project)
                    
    
    def generate(self, output):
        logging.info("start generation into %s" % (output,))
                
        Generator.generate(self, output)
        
        from pkg_resources import resource_filename
        template_dir = resource_filename(__name__, '/template/default')
        logging.info("found template dir " + template_dir)
        import os
        from os.path import join
        for root, dirs, files in os.walk(template_dir):
            # ignoring .svn directory
            svn_dir = ".svn"
            if svn_dir in dirs:
                dirs.remove(svn_dir)
            
            # apply transformation on each file
            for aFile in files:
                aTemplatePool = [self.templateFactory.get_template(template_dir, os.path.join(root,aFile))]
                self.generateTemplatePool(output, aTemplatePool)
            
            


${HEADER}

'''
Created on ${DATE_TIME}

@author: ${AUTHOR}
'''

import gtk
import hildon
import logging
import os.path
import logging
import time
import datetime
import pango

import urllib



from ${NAME}.gui.gtk.widget import  *
from ${NAME}.gui.hildon.widget import *
from ${NAME}.gui.gtk.utils import fill_widget_with_logo, LOGO_PIXBUF
from ${NAME}.gui.hildon.utils import show_about_dialog, call_handled_method, not_yet_implemented, \
                                    ${NAME_CAP}StackableWindow, ASYNC_EXCEPTION_HANDLER_INSTANCE, AsyncTask, \
                                    StopSignalException, show_banner_information, show_note_information
from ${NAME}.core import facade

from ${NAME}.common import version

version.getInstance().submitRevision("$Revision: 139 $")

from ${NAME}.gui.hildon.portrait import FremantleRotation


gtk.gdk.threads_init()


def show_new_window(window):
    program = hildon.Program.get_instance()
    program.add_window(window)
    window.show_all()


def show_album_list_view(facade):
        window = AlbumListView(facade)
        show_new_window(window)


def show_feed_list_view(facade):
        window = LastPictureView(facade)
        show_new_window(window)
      

class ${NAME}Gui(object):
    '''
    This is the GUI of gnatirac
    '''

    _last_folder = None

    def __init__(self):
        '''
        Create a new application GUI
        '''
        self.program = hildon.Program.get_instance()     
        self.facade = facade.${NAME}()       
        ASYNC_EXCEPTION_HANDLER_INSTANCE.start_async_exception_handler()
        self.init_main_view()   




    def init_main_view(self):
        '''
        create a new window for the main view of the application
        '''
       
        # display a spashscreen
        window = SplashScreenView(self.facade)
        window.connect("destroy", self.quit_application, None)
        show_new_window(window)
        
        # autorotayion !!!
        FremantleRotation("${NAME}",main_window=window)       

        # TODO do some stuff here       
        if self.facade.connected:
            # Show album list
            show_album_list_view(self.facade)
                             

    def quit_application(self, widget, data):
        ASYNC_EXCEPTION_HANDLER_INSTANCE.stop_async_exception_handler()
        gtk.main_quit()
        

        
      
        
    def run(self):
        # the splash screen  is displayed,now show the home screen in a fancy tansition               
        gtk.main()


      

    
class SplashScreenView(${NAME_CAP}StackableWindow):
    '''
    This is the first view of the application e.g. the main view.  
    '''

    def __init__(self, facade):
        self.facade = facade
        ${NAME_CAP}StackableWindow.__init__(self)

    def init_center_view(self, centerview):
        fill_widget_with_logo(centerview)

    


    def init_menu(self, menu):
        # add specific application menu here
        pass;
                                                                                                                                                          

 
        
        

                    
    


${HEADER}

'''
Created on ${DATE_TIME}

@author: ${AUTHOR}
'''

import ez_setup
ez_setup.use_setuptools()

from setuptools import setup, find_packages
setup(name='${NAME}',
      version='${VERSION}',
      package_dir = {'': 'src'},
      packages=find_packages(where='src'),               
      scripts=['scripts/${NAME}'],
      package_data={'${NAME}': ['*.png','*.jpg']},      
      data_files=[('/usr/share/applications/hildon',['hildon/${NAME}.desktop']),
                  ('/usr/share/icons/hicolor/48x48/hildon',['hildon/icons/48x48/${NAME}.png']),
                  ('/usr/share/icons/hicolor/64x64/hildon',['hildon/icons/64x64/${NAME}.png'])],    
      install_requires=["gdata>=2.0.9,<=2.0.14"],
      author='${AUTHOR}',
      author_email='${AUTHOR_EMAIL}',
      maintainer='${AUTHOR}',
      maintainer_email='${AUTHOR_EMAIL}',
      url='${PROJECT_URL}',
      download_url='${SITE_URL}',
      description='${SUMMARY}',
      long_description='${DESCRIPTION}',
      classifiers=[
          'Development Status :: 2 - Pre-Alpha',
          "Environment :: Handhelds/PDA's",          
          'Intended Audience :: End Users/Desktop',
          'Intended Audience :: Developers',          
          'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)',
          'Operating System :: POSIX :: Linux',
          'Programming Language :: Python'          
          ],
          zip_safe=False
   
      )




# -*- encoding: UTF-8 -*-

#    Pyjama is a python project bootstraper for Maemo5 development
#
#    Copyright (C) 2011  Thierry Bressure
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>

'''
Created on Dec 29, 2011

@author: maemo
'''

CURRENT_VERSION = "0.1.0-SNAPSHOT"  # this must be modified before any release

def getInstance():
    return SINGLETON

class Version(object):
    '''
    Hold the revision of the application.
    '''        

    def __init__(self):
        '''
        Constructor
        '''
        self.revision = ""
        
    def submitRevision(self, rev):
        '''
        submit a new revision. If the revision is upper than
        the current revision, it become the new current revision
        '''
        
        if self.revision < rev:
            self.revision = rev
            
    def getRevision(self):
        '''
        Return the scm revision for this application
        '''
        return self.revision
    
    def getVersion(self):
        '''
        Return the marketing version for this application.
        '''
        return CURRENT_VERSION
    
SINGLETON = Version()  # object that track the revision of current applcation for issue tracking purpose

getInstance().submitRevision("$Revision: 4 $")



# -*- encoding: UTF-8 -*-

#    Pyjama is a python project bootstraper for Maemo5 development
#
#    Copyright (C) 2011  Thierry Bressure
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>

'''
Created on Nov 16, 2011

@author: maemo
'''

import gtk


from pyjama.common import version

version.getInstance().submitRevision("$Revision: 37 $")


LOGO_PIXBUF = gtk.gdk.pixbuf_new_from_file("pyjama-logo.png")


def fill_widget_with_logo(centerview):
    pixbuf = LOGO_PIXBUF
    for i in range(1,3):
        hbox = gtk.HBox()
        for j in range(1,5):                
            image = gtk.Image()
            image.set_from_pixbuf(pixbuf)
            hbox.add(image)
        centerview.add(hbox)
        



                    


# -*- encoding: UTF-8 -*-

#    Pyjama is a python project bootstraper for Maemo5 development
#
#    Copyright (C) 2011  Thierry Bressure
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>

'''
Created on 20 Oct. 2011

@author: thierry
'''
import ez_setup
ez_setup.use_setuptools()

from setuptools import setup, find_packages
setup(name='pyjama',
      version='0.1.0',
      package_dir = {'': 'src'},
      packages=find_packages(where='src'),               
      scripts=['scripts/pyjama'],
      package_data={'pyjama': ['*.png','*.jpg']},      
      data_files=[('/usr/share/applications/hildon',['hildon/pyjama.desktop']),
                  ('/usr/share/icons/hicolor/48x48/hildon',['hildon/icons/48x48/pyjama.png']),
                  ('/usr/share/icons/hicolor/64x64/hildon',['hildon/icons/64x64/pyjama.png'])],    
      install_requires=["gdata>=2.0.9,<=2.0.14"],
      author='Thierry Bressure',
      author_email='thierry@bressure.net',
      maintainer='Thierry Bressure',
      maintainer_email='pyjama@bressure.net',
      url='http://blog.pyjama.bressure.net',
      download_url='http://pyjama.bressure.net',
      description='Pyjama is a python project bootstraper for Maemo5 development',
      long_description='Pyjama is a python project bootstraper for Maemo5 development. It generate the project structure ready for build and run',
      classifiers=[
          'Development Status :: 2 - Pre-Alpha',
          "Environment :: Handhelds/PDA's",          
          'Intended Audience :: End Users/Desktop',
          'Intended Audience :: Developers',          
          'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)',
          'Operating System :: POSIX :: Linux',
          'Programming Language :: Python'          
          ],
          zip_safe=False
   
      )




# -*- encoding: UTF-8 -*-

#    Pyjama is a python project bootstraper for Maemo5 development
#
#    Copyright (C) 2011  Thierry Bressure
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>
'''
Created on Oct 14, 2011

@author: maemo
'''




from optparse import OptionParser

import logging

from pyjama.common import version 


version.getInstance().submitRevision("$Revision: 4 $")


LEVELS = {'debug': logging.DEBUG,
          'info': logging.INFO,
          'warning': logging.WARNING,
          'error': logging.ERROR,
          'critical': logging.CRITICAL}




def showGui():
    from pyjama.gui.hildon.pyjamaGui import pyjamaGui 
    gui = pyjamaGui()
    gui.run()
    
def noGui(output=None):
    from pyjama.core import facade
    facade.pyjama().generate(output)
    
 

    
def run():
   versionManager = version.getInstance()
   usage = "%prog "
   str_version = "%prog " + versionManager.getVersion() + "(" + versionManager.getRevision() + ")"
   parser = OptionParser(usage=usage, version=str_version)
#
#   parser.add_option("-l","--login",action="store", dest="login", help="the login of the user without the @gmail")
#   parser.add_option("-p","--password",action="store", dest="password", help="the password for the given login")
#   parser.add_option("-d","--destination",action="store", dest="folder", help="the remote folder or album name")
#   parser.add_option("-r","--remove",action="store_true", dest="remove",  default=False, help="remove local file when uploaded")
#   parser.add_option("-f","--flatten",action="store_true", dest="flatten", default=False, help="subfolder are flattened to folder with name prefixed by given -d option value if any")
#   parser.add_option("-v","--verbose",action="store_true", dest="verbose", default=False, help="print verbose output of saving progress")
   parser.add_option("-o","--output", action="store", dest="output", default=".", help="output location")
   parser.add_option("-g","--gui",action="store_true", dest="gui", default=False, help="show the gui")   
   parser.add_option("-l", "--log", action="store", type="string", dest="level_name", help="log level")
      
   (options, args) = parser.parse_args()
   
   level = LEVELS.get(options.level_name, logging.NOTSET)
   logging.basicConfig(level=level)

   if options.gui:   
       showGui()
   else:
       noGui(options.output)
    




if __name__ == '__main__':  
    run()
